#
# Author: Facundo Batista <facundo@canonical.com>
# Author: Guillermo Gonzalez <guillermo.gonzalez@canonical.com>
#
# Copyright 2009, 2010 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Mute Filter."""

import unittest

from ubuntuone.syncdaemon.mute_filter import MuteFilter


class MuteFilterTests(unittest.TestCase):
    """Tests the MuteFilter class."""

    def setUp(self):
        self.mf = MuteFilter()

    def test_empty(self):
        """Nothing there."""
        self.assertFalse(self.mf._cnt)

    def test_add_one(self):
        """Adds one element."""
        self.mf.add("foo")
        self.assertEqual(self.mf._cnt, dict(foo=1))

    def test_add_two_different(self):
        """Adds two different elements."""
        self.mf.add("foo")
        self.mf.add("bar")
        self.assertEqual(self.mf._cnt, dict(foo=1, bar=1))

    def test_add_two_equal(self):
        """Adds one element twice."""
        self.mf.add("foo")
        self.mf.add("foo")
        self.assertEqual(self.mf._cnt, dict(foo=2))

    def test_add_two_equal_and_third(self):
        """Adds one element."""
        self.mf.add("foo")
        self.mf.add("bar")
        self.mf.add("bar")
        self.assertEqual(self.mf._cnt, dict(foo=1, bar=2))

    def test_pop_simple(self):
        """Pops one element."""
        self.mf.add("foo")
        self.assertFalse(self.mf.pop("bar"))
        self.assertEqual(self.mf._cnt, dict(foo=1))
        self.assertTrue(self.mf.pop("foo"))
        self.assertFalse(self.mf._cnt)

    def test_pop_complex(self):
        """Pops several elements."""
        # add several
        self.mf.add("foo")
        self.mf.add("bar")
        self.mf.add("bar")
        self.assertEqual(self.mf._cnt, dict(foo=1, bar=2))

        # clean bar
        self.assertTrue(self.mf.pop("bar"))
        self.assertEqual(self.mf._cnt, dict(foo=1, bar=1))
        self.assertTrue(self.mf.pop("bar"))
        self.assertEqual(self.mf._cnt, dict(foo=1))
        self.assertFalse(self.mf.pop("bar"))
        self.assertEqual(self.mf._cnt, dict(foo=1))

        # clean foo
        self.assertTrue(self.mf.pop("foo"))
        self.assertFalse(self.mf._cnt)
        self.assertFalse(self.mf.pop("foo"))
        self.assertFalse(self.mf._cnt)



