# ubuntuone.syncdaemon.mute_filter - Mute Filter
#
# Author: Facundo Batista <facundo@canonical.com>
#
# Copyright 2009, 2010 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Class that filters and mutes some events on some paths."""

import logging

class MuteFilter(object):
    """Stores what needs to be muted."""
    def __init__(self):
        self._cnt = {}
        self.log = logging.getLogger('ubuntuone.SyncDaemon.MuteFilter')

    def add(self, element):
        """Add and element to the filter."""
        self.log.debug("Adding: %s", element)
        self._cnt[element] = self._cnt.get(element, 0) + 1

    def rm(self, element):
        """Remove an element from the filter."""
        self.log.debug("Removing: %s", element)
        new_val = self._cnt[element] - 1
        if new_val == 0:
            del self._cnt[element]
        else:
            self._cnt[element] = new_val

    def pop(self, element):
        """Pops an element from the filter, if there, and returns if it was."""
        if element not in self._cnt:
            return False

        self._cnt[element] = self._cnt.get(element, 0) - 1
        if not self._cnt[element]:
            # reached zero
            del self._cnt[element]

        # log what happened and how many items we have left
        q = sum(self._cnt.itervalues())
        self.log.debug("Blocking %s (%d left)", element, q)

        return True
